"""
    file        sc_restart_dfu.py
    brief       Restart an H663 device in DFU mode
    author      Matthieu Barreteau <matthieu.b@springcard.com>
    date        22/06/2021
 
    copyright   Copyright (c) 2021 SPRINGCARD SAS, FRANCE - www.springcard.com
    version     MBA 22/06/2021: initial release

    Requirements:
    - Python >= 3.7.3
    - Pyscard (python -m pip install pyscard)

    Purpose:
    The aim of this project is to restart an H663 in
	DFU mode to perform a firmware upgraede

    """
	
# libraries import
import binascii
import logging
import struct
import getopt
import time
import sys
import os

# helpers import
from smartcard.System import *
from smartcard.pcsc.PCSCPart10 import (SCARD_SHARE_DIRECT, \
                                        SCARD_LEAVE_CARD, \
                                        FEATURE_CCID_ESC_COMMAND, \
                                        getFeatureRequest, \
                                        hasFeature)
from smartcard.util import toHexString

# display_help function
def display_help( script_name ):

    print( f"{script_name} [--index=coupler number] [--list] [--getversion]" )
    sys.exit()


# main function  
def main( script_name, argv ):
    
    # default settings
    coupler_index = 0
    working_path = os.getcwd()
    show_list = False
    get_version = False

    # options parsing
    try:
        opts, _ = getopt.getopt(argv,"h",[ "help", "index=", "list", "getversion" ])
    except getopt.GetoptError as e:
        print(e)
        display_help( script_name )

    for opt, arg in opts:
        if opt in ( "-h", "--help"):
            display_help( script_name )        
        elif opt == "--list":
            show_list = True
        elif opt == "--getversion":
            get_version = True     
        elif opt == "--index":
            try:
                coupler_index = int(arg)
            except ValueError:
                print( "Index must be an integer number!" )
                sys.exit()

    # set debug options
    logging.basicConfig(format='%(asctime)s - %(message)s', \
                        datefmt='%d-%b-%y %H:%M:%S', \
                        level=logging.DEBUG )  

    # get coupler list
    couplers = readers()
    if not len( couplers):
        logging.error( "No coupler found!" )
        sys.exit()

    if show_list:
        id = 0
        logging.info( "Available couplers:" )
        for acoupler in couplers:
            print( f"{id}: {acoupler}")
            id += 1
        sys.exit()

    # Connect to this coupler
    logging.info( f"connecting to {couplers[ coupler_index ]}" )
    
    try:
        link = couplers[ coupler_index ].createConnection()
        link.connect()
    except:
        logging.error( "ERROR: Can't connect to this coupler, please place a card on it!" )
        sys.exit()

    # Check if PC_to_RDR_Escape commands are usable
    coupler_features = getFeatureRequest(link)
    PC_to_RDR_Escape = hasFeature( coupler_features, FEATURE_CCID_ESC_COMMAND )
    if PC_to_RDR_Escape is None:
        logging.error( "ERROR: PC_to_RDR_Escape command is not usable, " \
                        "please check your Info.plist file (ifdDriverOptions)!" )
        link.disconnect()
        sys.exit()

    # Get current firmware version
    APDU_GET_VERSION = [ 0x58, 0x20, 0x05 ]
    data = link.control( PC_to_RDR_Escape, APDU_GET_VERSION )    
    if len(data) <= 0:
        logging.error( "ERROR: Can't get current firmware version!" )
        link.disconnect()
        sys.exit()

    firmware_version = "".join(map(chr,data[1:]))
    logging.info( f"Current firmware version is {firmware_version}!" )
    if get_version:
        link.disconnect()
        sys.exit()

    logging.info( "Restarting in DFU!" )
    APDU_RESTART_IN_DFU_MODE = [ 0x9F, 0xFA, 0xDA, 0xFA, 0xDA ]
    try:
        data = link.control( PC_to_RDR_Escape, APDU_RESTART_IN_DFU_MODE )
    except:
        # this is a legit error, the device has restarted!
        pass

    logging.info( "Done." )
    print( "Your device should be in DFU mode, you can now use dfu-programmer to upgrade it with:")
    print( "dfu-programmer at32uc3b0256 erase" )
    print( "dfu-programmer at32uc3b0256 flash --suppress-bootloader-mem uc3b0256_rc663_h663_2-40.hex" )
    print( "dfu-programmer at32uc3b0256 start" )


# entry point
if __name__ == "__main__":
    # add "this script" folder to the library search path
    # could be useful on Windows platform
    sys.path.append( os.path.dirname( os.path.realpath( __file__ ) ) )
  
    # start our application
    main( os.path.basename( __file__ ), sys.argv[1:] )

# EOF